from .gen.organization_exports import _OrganizationExports


class OrganizationExports(_OrganizationExports):
    """The :class:`OrganizationExports` object, which represents the resource
    of the same name in Asana's API.

    """
